@@ -1,9 +1,27 @@
 #ifndef __CR_PID_H__
 #define __CR_PID_H__
 
+#include <compel/task-state.h>
 #include "stdbool.h"
 #include "rbtree.h"
 
+/*
+ * Task states, used in e.g. struct pid's state.
+ */
+enum __criu_task_state
+{
+	/* Values shared with compel */
+	TASK_ALIVE		= COMPEL_TASK_ALIVE,
+	TASK_DEAD		= COMPEL_TASK_DEAD,
+	TASK_STOPPED		= COMPEL_TASK_STOPPED,
+	TASK_ZOMBIE		= COMPEL_TASK_ZOMBIE,
+	/* Own internal states */
+	TASK_HELPER		= COMPEL_TASK_MAX + 1,
+	TASK_THREAD,
+	/* new values are to be added before this line */
+	TASK_UNDEF		= 0xff
+};
+
 struct pid {
 	struct pstree_item *item;
 	/*
@@ -26,14 +44,6 @@ struct pid {
 	} ns[1]; /* Must be at the end of struct pid */
 };
 
-#define TASK_UNDEF		0x0
-#define TASK_ALIVE		0x1
-#define TASK_DEAD		0x2
-#define TASK_STOPPED		0x3
-#define TASK_HELPER		0x4
-#define TASK_THREAD		0x5
-#define TASK_ZOMBIE		0x6
-
 /*
  * When we have to restore a shared resource, we mush select which
  * task should do it, and make other(s) wait for it. In order to
