@@ -17,6 +17,7 @@
 #include "images/pipe.pb-c.h"
 #include "images/pipe-data.pb-c.h"
 #include "fcntl.h"
+#include "namespaces.h"
 
 static LIST_HEAD(pipes);
 
@@ -213,10 +214,10 @@ err:
 	return ret;
 }
 
-static int reopen_pipe(int fd, int flags)
+static int userns_reopen(void *_arg, int fd, pid_t pid)
 {
-	int ret;
 	char path[PSFDS];
+	int ret, flags = *(int*)_arg;
 
 	sprintf(path, "/proc/self/fd/%d", fd);
 	ret = open(path, flags);
@@ -227,23 +228,36 @@ static int reopen_pipe(int fd, int flags)
 	return ret;
 }
 
+static int reopen_pipe(int fd, int flags)
+{
+	int ret;
+	char path[PSFDS];
+
+	sprintf(path, "/proc/self/fd/%d", fd);
+	ret = open(path, flags);
+	if (ret < 0) {
+		if (errno == EACCES) {
+			/* It may be an external pipe from an another userns */
+			ret = userns_call(userns_reopen, UNS_FDOUT,
+						&flags, sizeof(flags), fd);
+		} else
+			pr_perror("Unable to reopen the pipe %s", path);
+	}
+	close(fd);
+
+	return ret;
+}
+
 static int recv_pipe_fd(struct pipe_info *pi, int *new_fd)
 {
-	struct fdinfo_list_entry *fle;
 	int tmp, fd, ret;
 
-	fle = file_master(&pi->d);
-	fd = fle->fe->fd;
-
-	pr_info("\tWaiting fd for %d\n", fd);
-
-	ret = recv_fd_from_peer(fle);
+	ret = recv_desc_from_peer(&pi->d, &tmp);
 	if (ret != 0) {
 		if (ret != 1)
-			pr_err("Can't get fd %d\n", fd);
+			pr_err("Can't get fd %d\n", tmp);
 		return ret;
 	}
-	tmp = fd;
 
 	if (pi->reopen)
 		fd = reopen_pipe(tmp, pi->pe->flags);
@@ -304,13 +318,9 @@ int open_pipe(struct file_desc *d, int *new_fd)
 		return -1;
 
 	list_for_each_entry(p, &pi->pipe_list, pipe_list) {
-		struct fdinfo_list_entry *fle;
-		int fd;
+		int fd = pfd[p->pe->flags & O_WRONLY];
 
-		fle = file_master(&p->d);
-		fd = pfd[p->pe->flags & O_WRONLY];
-
-		if (send_fd_to_peer(fd, fle)) {
+		if (send_desc_to_peer(fd, &p->d)) {
 			pr_perror("Can't send file descriptor");
 			return -1;
 		}
@@ -362,9 +372,8 @@ int collect_one_pipe_ops(void *o, ProtobufCMessage *base, struct file_desc_ops *
 			list_add(&pi->pipe_list, &tmp->pipe_list);
 	}
 
-	if (list_empty(&pipes))
-		if (add_post_prepare_cb(mark_pipe_master, NULL))
-			return -1;
+	if (add_post_prepare_cb_once(mark_pipe_master, NULL))
+		return -1;
 
 	list_add_tail(&pi->list, &pipes);
 
