@@ -1,13 +1,11 @@
+#include <errno.h>
+#include <libnet.h>
+#include <linux/sockios.h>
+#include <linux/types.h>
 #include <netinet/tcp.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/ioctl.h>
-#include <errno.h>
-#include <linux/sockios.h>
-#include <libnet.h>
-#include <assert.h>
-#include <errno.h>
-
 #include "soccr.h"
 
 #ifndef SIOCOUTQNSD
@@ -16,17 +14,17 @@
 #endif
 
 enum {
-        TCPF_ESTABLISHED = (1 << 1),
-        TCPF_SYN_SENT    = (1 << 2),
-        TCPF_SYN_RECV    = (1 << 3),
-        TCPF_FIN_WAIT1   = (1 << 4),
-        TCPF_FIN_WAIT2   = (1 << 5),
-        TCPF_TIME_WAIT   = (1 << 6),
-        TCPF_CLOSE       = (1 << 7),
-        TCPF_CLOSE_WAIT  = (1 << 8),
-        TCPF_LAST_ACK    = (1 << 9),
-        TCPF_LISTEN      = (1 << 10),
-        TCPF_CLOSING     = (1 << 11),
+	TCPF_ESTABLISHED = (1 << 1),
+	TCPF_SYN_SENT    = (1 << 2),
+	TCPF_SYN_RECV    = (1 << 3),
+	TCPF_FIN_WAIT1   = (1 << 4),
+	TCPF_FIN_WAIT2   = (1 << 5),
+	TCPF_TIME_WAIT   = (1 << 6),
+	TCPF_CLOSE       = (1 << 7),
+	TCPF_CLOSE_WAIT  = (1 << 8),
+	TCPF_LAST_ACK    = (1 << 9),
+	TCPF_LISTEN      = (1 << 10),
+	TCPF_CLOSING     = (1 << 11),
 };
 
 /*
@@ -76,7 +74,7 @@ void libsoccr_set_log(unsigned int level, void (*fn)(unsigned int level, const c
 	log = fn;
 }
 
-#define loge(msg, ...) do { if (log && (log_level >= SOCCR_LOG_ERR)) log(SOCCR_LOG_ERR, "Error: " msg, ##__VA_ARGS__); } while (0)
+#define loge(msg, ...) do { if (log && (log_level >= SOCCR_LOG_ERR)) log(SOCCR_LOG_ERR, "Error (%s:%d): " msg, __FILE__, __LINE__, ##__VA_ARGS__); } while (0)
 #define logerr(msg, ...) loge(msg ": %s\n", ##__VA_ARGS__, strerror(errno))
 #define logd(msg, ...) do { if (log && (log_level >= SOCCR_LOG_DBG)) log(SOCCR_LOG_DBG, "Debug: " msg, ##__VA_ARGS__); } while (0)
 
@@ -121,8 +119,10 @@ struct libsoccr_sk *libsoccr_pause(int fd)
 	struct libsoccr_sk *ret;
 
 	ret = malloc(sizeof(*ret));
-	if (!ret)
+	if (!ret) {
+		loge("Unable to allocate memory\n");
 		return NULL;
+	}
 
 	if (tcp_repair_on(fd) < 0) {
 		free(ret);
@@ -321,8 +321,10 @@ static int get_queue(int sk, int queue_id,
 		 * make sure there are len bytes for real
 		 */
 		buf = malloc(len + 1);
-		if (!buf)
+		if (!buf) {
+			loge("Unable to allocate memory\n");
 			goto err_buf;
+		}
 
 		ret = recv(sk, buf, len + 1, MSG_PEEK | MSG_DONTWAIT);
 		if (ret != len)
@@ -353,8 +355,10 @@ int libsoccr_save(struct libsoccr_sk *sk, struct libsoccr_sk_data *data, unsigne
 {
 	struct tcp_info ti;
 
-	if (!data || data_size < SOCR_DATA_MIN_SIZE)
+	if (!data || data_size < SOCR_DATA_MIN_SIZE) {
+		loge("Invalid input parameters\n");
 		return -1;
+	}
 
 	memset(data, 0, data_size);
 
@@ -443,16 +447,22 @@ static int libsoccr_set_sk_data_noq(struct libsoccr_sk *sk,
 	int onr = 0;
 	__u32 seq;
 
-	if (!data || data_size < SOCR_DATA_MIN_SIZE)
+	if (!data || data_size < SOCR_DATA_MIN_SIZE) {
+		loge("Invalid input parameters\n");
 		return -1;
+	}
 
-	if (!sk->dst_addr || !sk->src_addr)
+	if (!sk->dst_addr || !sk->src_addr) {
+		loge("Destination or/and source addresses aren't set\n");
 		return -1;
+	}
 
 	mstate = 1 << data->state;
 
-	if (data->state == TCP_LISTEN)
+	if (data->state == TCP_LISTEN) {
+		loge("Unable to handle listen sockets\n");
 		return -1;
+	}
 
 	if (sk->src_addr->sa.sa_family == AF_INET)
 		addr_size = sizeof(sk->src_addr->v4);
@@ -460,7 +470,7 @@ static int libsoccr_set_sk_data_noq(struct libsoccr_sk *sk,
 		addr_size = sizeof(sk->src_addr->v6);
 
 	if (bind(sk->fd, &sk->src_addr->sa, addr_size)) {
-		loge("Can't bind inet socket back\n");
+		logerr("Can't bind inet socket back");
 		return -1;
 	}
 
@@ -561,14 +571,18 @@ static int send_fin(struct libsoccr_sk *sk, struct libsoccr_sk_data *data,
 		libnet_type = LIBNET_RAW4;
 
 	l = libnet_init(
-		libnet_type,                            /* injection type */
-		NULL,                                   /* network interface */
-		errbuf);                                /* errbuf */
-	if (l == NULL)
+		libnet_type,		/* injection type */
+		NULL,			/* network interface */
+		errbuf);		/* errbuf */
+	if (l == NULL) {
+		loge("libnet_init failed (%s)\n", errbuf);
 		return -1;
+	}
 
-	if (setsockopt(l->fd, SOL_SOCKET, SO_MARK, &mark, sizeof(mark)))
+	if (setsockopt(l->fd, SOL_SOCKET, SO_MARK, &mark, sizeof(mark))) {
+		logerr("Can't set SO_MARK (%d) for socket\n", mark);
 		goto err;
+	}
 
 	ret = libnet_build_tcp(
 		ntohs(sk->dst_addr->v4.sin_port),		/* source port */
@@ -622,7 +636,7 @@ static int send_fin(struct libsoccr_sk *sk, struct libsoccr_sk_data *data,
 			l,			/* libnet handle */
 			0);			/* libnet id */
 	else {
-		loge("Unknown socket family");
+		loge("Unknown socket family\n");
 		goto err;
 	}
 	if (ret == -1) {
@@ -632,7 +646,7 @@ static int send_fin(struct libsoccr_sk *sk, struct libsoccr_sk_data *data,
 
 	ret = libnet_write(l);
 	if (ret == -1) {
-		loge("Unable to send a fin packet: %s", libnet_geterror(l));
+		loge("Unable to send a fin packet: %s\n", libnet_geterror(l));
 		goto err;
 	}
 
